{% if file['file'] %} {% if config.write_namer_failed_log %} <button type="button" class="btn btn-secondary log" data-bs-toggle="modal" data-bs-target="#logFile" data-file="{{ file['file'] }}" title="Show Log"><i class="bi bi-info-circle"></i></button> {% endif %} <button type="button" class="btn btn-primary match" data-bs-toggle="modal" data-bs-target="#searchForm" data-file="{{ file['file'] }}" data-query="{{ file['name'] }}" title="Search"><i class="bi bi-search"></i></button> {% if config.allow_delete_files %} <button type="button" class="btn btn-danger delete" data-bs-toggle="modal" data-bs-target="#deleteConfirm" data-file="{{ file['file'] }}" title="Delete"><i class="bi bi-trash"></i></button> {% endif %} {% endif %}