{% extends "partials/base.html" %} {% block title %}Failed files{% endblock %} {% set active_page = "failed" %} {% block content %} <h5 class="card-title d-flex justify-content-between align-items-center">Unmatched Files <button id="refreshFiles" type="button" class="btn btn-secondary ms-1" data-target="files" title="Refresh"><i class="bi bi-arrow-clockwise"></i></button></h5><div class="card-text"><div id="filesResult"> {% include 'render/failedFiles.html' %} </div></div> {% endblock %} {% block external %} {% if config.write_namer_failed_log %} <div id="logFile" class="modal fade" tabindex="-1" aria-labelledby="modalLogsLabel" aria-hidden="true"><div class="modal-dialog modal-xl"><div class="modal-content"><div class="modal-header"><h5 class="modal-title" id="modalLogsLabel">Failed Log</h5><button type="button" class="btn-close" data-bs-dismiss="modal" aria-label="Close"></button></div><div class="modal-body"><div class="progress"></div></div><div class="modal-footer"><button type="button" class="btn btn-secondary" data-bs-dismiss="modal">Close</button></div></div></div></div> {% endif %} <div id="searchForm" class="modal fade" tabindex="-1" aria-labelledby="searchFormLabel" aria-hidden="true"><div class="modal-dialog"><div class="modal-content"><div class="modal-header"><h5 class="modal-title" id="searchFormLabel">Search</h5><button type="button" class="btn-close" data-bs-dismiss="modal" aria-label="Close"></button></div><div class="modal-body"><label for="queryInput" class="form-label">Search query</label> <input type="text" class="form-control" id="queryInput"></div><div class="modal-footer"><button type="button" class="btn btn-secondary" data-bs-dismiss="modal">Close</button> <button type="button" class="btn btn-primary search" data-bs-target="#searchResults" data-bs-toggle="modal" data-bs-dismiss="modal">Submit</button></div></div></div></div><div id="searchResults" class="modal fade" tabindex="-1" aria-labelledby="modalSearchResultsLabel" aria-hidden="true"><div class="modal-dialog modal-xl modal-fullscreen-lg-down"><div class="modal-content"><div class="modal-header"><h5 class="modal-title" id="modalSearchResultsLabel">Search Result</h5><button type="button" class="btn-close" data-bs-dismiss="modal" aria-label="Close"></button></div><div class="modal-body"><div class="progress"></div></div><div class="modal-footer"><button type="button" class="btn btn-secondary" data-bs-target="#searchForm" data-bs-toggle="modal" data-bs-dismiss="modal">Back</button> <button type="button" class="btn btn-secondary" data-bs-dismiss="modal">Close</button></div></div></div></div> {% if config.allow_delete_files %} <div id="deleteConfirm" class="modal fade" tabindex="-1" aria-labelledby="modalDeleteConfirmLabel" aria-hidden="true"><div class="modal-dialog"><div class="modal-content"><div class="modal-header"><h5 class="modal-title" id="modalDeleteConfirmLabel">Confirmation</h5><button type="button" class="btn-close" data-bs-dismiss="modal" aria-label="Close"></button></div><div class="modal-body"><p>Are you sure you wanted to delete this file?</p><label for="deleteFile" class="form-label">File</label> <input id="deleteFile" type="text" class="form-control" disabled="disabled"></div><div class="modal-footer"><button type="button" class="btn btn-secondary" data-bs-dismiss="modal">Close</button> <button id="deleteButton" type="button" class="btn btn-danger" data-bs-dismiss="modal">Yes</button></div></div></div></div> {% endif %} {% endblock %}