{% if data %} <table id="{{ active_page }}" class="table table-sm text-start align-middle"><thead><tr><th scope="col">File Name</th><th scope="col">Extension</th><th scope="col">Size</th><th scope="col" data-orderable="false"></th></tr></thead><tbody> {% for file in data %} <tr><td class="col">{{ file['name'] }}</td><td class="col-1">{{ file['ext'] }}</td><td class="col-1" data-order="{{ file['size_byte'] }}">{{ file['size'] }}</td><td class="col-2 text-end"> {% include 'render/fileActions.html' %} </td></tr> {% endfor %} </tbody></table> {% else %} <div class="col m-1 text-center"><span>No files found</span></div> {% endif %}